/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;

namespace Starter {
    public class Subscriber {
        public static void Main() {
            Publisher publisher = new Publisher();
            // Subscribe to events and set method reference.
            publisher.BeginOutput += StartOutputCallback;
            publisher.EndOutput   += EndOutputCallback;
            publisher.Display("I am a subscriber.");

            // De-reference callback methods.
            publisher.BeginOutput -= StartOutputCallback;
            publisher.EndOutput   -= EndOutputCallback;

            // Output text when not subscribed to the event.
            publisher.Display("\nI am not a subscriber.");
            Console.ReadLine();
        }

        public static void StartOutputCallback(string message) {
            Console.WriteLine("StartOutputCallback - " + message);
        }

        public static void EndOutputCallback(string message) {
            Console.WriteLine("EndOutputCallback - " + message);
        }
    }

    public class Publisher {
        public delegate void Notify(string message); // Declare delegate.
        public event Notify BeginOutput;             // Declare event.
        public event Notify EndOutput;               // Declare event.

        public void Display(string message) {
            OnBeginOutput();                         // Raise event.
            Console.WriteLine(message);              // Show output.
            OnEndOutput();                           // Raise event.
        }

        private void OnBeginOutput() {
            if (BeginOutput != null)
                BeginOutput("Starting output!");     // Notify callback method.
        }

        private void OnEndOutput() {
            if (EndOutput != null)
                EndOutput("Ending output!");         // Notify callback method.
        }
    }
}
