/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;

namespace ConsoleApplication1 {
    class Program {
        private static void Main() {
            FoodStoreEntities context = new FoodStoreEntities(); // Reference database.
            
            // Outer join on Stores.
            var allStores = from s in context.Stores             // Outer table.
                from i in context.Invoices                       // Inner table.
                   .Where(inv => inv.branch == s.branch)         // Join columns.
                   .DefaultIfEmpty()
                // Set null for inner table if no match.
                select new {
                    Branch     = s.branch,
                    Region     = s.region,
                    // Store invoiceNum value if it exists otherwise assign 0.
                    InvoiceNum = (i.invoiceNum != null) ? i.invoiceNum : 0 };

            // Outer join on Invoices.
            var allInvoices = from i in context.Invoices         // Outer table.
                from s in context.Stores                         // Inner table.
                    .Where(store => store.branch == i.branch)    // Join columns.
                    .DefaultIfEmpty()
                // Set null for inner table if no match.
                select new {
                    // Set branch and region if values exist. Otherwise assign null.
                    Branch = (s.branch != null) ? s.branch : "null",
                    Region = (s.region != null) ? s.region : "null",
                    InvoiceNum = i.invoiceNum };

            // Full outer join.
            var fullJoin = allStores.Union(allInvoices);
            foreach (var item in fullJoin)                       // Display results.
                Console.WriteLine(item.Branch + " - " + item.Region + " - "
                                + item.InvoiceNum);
            Console.ReadLine();
        }
    }
}
