using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;

namespace Starter.DataAccessLayer {
    public class DBEngine {
        // Build the C# adapter for the database command and connection.
        private static SqlDataAdapter Adapter(string command) {
            SqlDataAdapter adapter = null;
            try {
                string connection =
                    ConfigurationManager.ConnectionStrings["MyConnection"]
                                        .ConnectionString;
                adapter = new SqlDataAdapter(command, connection);
                adapter.SelectCommand.CommandTimeout = 10;
            }
            catch (Exception e) {
                Console.WriteLine("A database connection error occurred. "
                                + "Please check your App.config settings: "
                                + e.Message);
            }
            return adapter;
        }

        // Add parameter names and values to the stored procedure adapter.
        private static SqlDataAdapter AddParameters(SqlDataAdapter adapter,
            string[] parameters, string[] values) {
            if (parameters == null && values == null)
                return adapter;
            else if (parameters.Length != values.Length) {
                Console.WriteLine("Error: " +
                                  "Total procedure parameter names and values do not match");
                return null;
            }
            for (int i = 0; i < values.Length; i++) {
                adapter.SelectCommand.Parameters.Add(
                    new SqlParameter(parameters[i], values[i]));
            }
            return adapter;
        }

        // Build a SqlDataAdapter for a stored procedure.
        private static SqlDataAdapter ProcedureAdapter(string procedure,
            string[] parameters, string[] values) {
            SqlDataAdapter adapter = Adapter(procedure);
            if (adapter == null)
                return null;
            adapter.SelectCommand.CommandType = CommandType.StoredProcedure;
            adapter = AddParameters( adapter, parameters, values);
            return adapter;
        }

        // Run the SQL or stored procedure and store results in a DataTable
        private static DataTable RunCommand(SqlDataAdapter adapter) {
            if (adapter == null)
                return null;
            DataTable table = new DataTable();
            try {
                adapter.Fill(table);
            }
            catch (Exception exception) {
                Console.Write("An error occurred while running the ");
                Console.Write("SQL or stored procedure: ");
                Console.WriteLine(exception.Message);
                return null;
            }
            return table;
        }

        // Execute stored procedure that has no parameters.
        public static DataTable ExecParameterlessProcedure(
            string procedure) {
            return RunCommand(ProcedureAdapter(procedure, null, null));
        }

        // Execute stored procedure that has parameters.
        public static DataTable ExecProcedure(string name,
            string[] parameters, string[] values) {
            return RunCommand(ProcedureAdapter(name, parameters, values));
        }

        // Execute SQL.
        public static DataTable ExecSQL(string sql) {
            return RunCommand(Adapter(sql));
        }
    }
}
