/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;
using System.Data.SqlClient;
using System.Configuration;

namespace Starter {
    class Reader {
        private SqlConnection sqlConnection;
        public  SqlDataReader Sdr { get; private set; }

        // Build SqlCommand for SQL only and run it.
        public void RunSQL(string sql) {
            Sdr = ExecuteReader(new SqlCommand(sql, OpenConnection()));
        }

        // Run the SqlDataReader and show errors if failure occurs.
        public SqlDataReader ExecuteReader(SqlCommand command) {
            try {
                return command.ExecuteReader();
            }
            catch (Exception e) {
                Console.WriteLine(e.Message);
            }
            return null;
        }

        // Open the connection and show errors if failure occurs.
        public SqlConnection OpenConnection() {
            try {
                string connection = ConfigurationManager
                                   .ConnectionStrings["MyConnection"].ConnectionString;
                sqlConnection = new SqlConnection(connection);
                sqlConnection.Open();
            }
            catch (Exception e) {
                Console.WriteLine("Data connection failed: "
                                + "Check the App.config reference. "
                                + e.Message);
            }
            return sqlConnection;
        }

        public void CloseReaderAndConnection() {
            if (Sdr != null)
                Sdr.Close();
            if (sqlConnection != null)
                sqlConnection.Close();
        }
    }

    class Program {
        public static void Main() {
            Reader reader = new Reader();
            reader.RunSQL("SELECT * FROM Product");
            ShowOutput(reader);
            Console.ReadLine();
        }

        private static void ShowOutput(Reader reader) {
            // Retrieve and display rows from database one-by-one.
            while (reader.Sdr != null && reader.Sdr.Read()) {
                // Reference database columns by name.
                int id             = (int) reader.Sdr["productID"];
                string productName = (string) reader.Sdr["name"];
                decimal      price = (decimal) reader.Sdr["price"];
                Console.WriteLine("ID: " + id + " Name: " + productName
                                + " Price: " + price.ToString("C"));
            }
            // Close the reader so you can reopen and use it later.
            if (reader != null)
                reader.CloseReaderAndConnection();
        }
    }
}
