using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

namespace Starter.DataAccessLayer {
    public class ReaderEngine {
        private SqlConnection sqlConnection;
        // Open the connection.
        private SqlConnection OpenConnection() {
            try {
                string connection = ConfigurationManager
                                   .ConnectionStrings["MyConnection"]
                                   .ConnectionString;
                sqlConnection = new SqlConnection(connection);
                sqlConnection.Open();
            }
            catch (Exception e) {
                Console.Write("An error occurred with the data ");
                Console.Write("connection. Please check the App.config");
                Console.Write(" reference and ensure the connection ");
                Console.WriteLine("is already open " + e.Message);
                return null;
            }
            return sqlConnection;
        }

        // Close database connection.
        public void CloseConnection() {
            if (sqlConnection != null)
                sqlConnection.Close();
        }

        // Build SqlCommand for stored procedure.
        private SqlCommand ProcedureCommand(string procedure,
            string[] parameterNames,
            string[] parameterValues) {
            SqlCommand command = new SqlCommand(procedure,
                OpenConnection());
            command.CommandType = CommandType.StoredProcedure;

            // Return SqlCommand object as is for parameterless procedure.
            if (parameterNames == null && parameterValues == null)
                return command;

            // Add parameters to SqlCommand for procedures with parameters.
            for (int i = 0; i < parameterValues.Length; i++) {
                command.Parameters.Add(
                    new SqlParameter(parameterNames[i],
                        parameterValues[i]));
            }
            return command;
        }

        // Execute the command.
        private SqlDataReader ExecuteReader(SqlCommand command) {
            try {
                return command.ExecuteReader();
            }
            // If there are errors show them.
            catch (Exception e) {
                Console.WriteLine(e.Message);
            }
            return null;
        }

        // Build SqlCommand for SQL only and run it.
        public SqlDataReader ExecSQL(string sqlString) {
            return ExecuteReader(new SqlCommand(sqlString,
                OpenConnection()));
        }

        // Execute procedure without parameters.
        public SqlDataReader ExecParameterlessProcedure(string procedure) {
            SqlCommand command = ProcedureCommand(procedure, null, null);
            return ExecuteReader(command);
        }

        // Execute procedure with parameters.
        public SqlDataReader ExecProcedure(string procedure,
            string[] parameterNames,
            string[] parameterValues) {
            SqlCommand command = ProcedureCommand(procedure,
                parameterNames,
                parameterValues);
            return ExecuteReader(command);
        }
    }
}
