/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;
using System.IO;

namespace Starter {
    class Program {
        private const string FILE_PATH = "../../MyBinaryBuffer.dat";
        private const int  BUFFER_SIZE = 16; // 16 bytes - 4 bytes per integer.

        public static void Main() {
            GenerateBinaryContent();
            ReadBinaryData();
            Console.ReadLine();
        }

        // Generates data and stores it in a file in binary format.
        private static void GenerateBinaryContent() {
            try {
                using (BinaryWriter writer
                    = new BinaryWriter(File.Open(FILE_PATH, FileMode.Create))) {
                    int[] numberArray = {1, 2, 3, 4, 5, 6, 7, 2, 1};
                    for (int i = 0; i < numberArray.Length; i++)
                        writer.Write(numberArray[i]);
                }
            }
            catch (Exception e) {
                Console.WriteLine(e.Message);
            }
        }

        // Reads binary data into 4 byte blocks.
        private static void ReadBinaryData() {
            byte[] buffer = new byte[BUFFER_SIZE];
            try {
                using (FileStream fileStream = new FileStream(FILE_PATH, FileMode.Open)) {
                    int bytesRead      = 0;
                    int totalBytesRead = 0;

                    // Read data into byte buffer as long as data exists.
                    while ((bytesRead
                        = fileStream.Read(buffer, 0, buffer.Length)) > 0) {
                        totalBytesRead += bytesRead;
                        ShowBufferContents(buffer);
                        buffer = ResetBuffer((int) fileStream.Length,
                            totalBytesRead);
                        if (buffer == null)
                            break;
                    }
                }
            }
            catch (Exception e) {
                Console.WriteLine("Read error: " + e.Message);
            }
        }

        // Extract and show data currently in the buffer.
        private static void ShowBufferContents(byte[] buffer) {
            const int BYTES_PER_INT = 4; // 1 integer = 4 bytes
            // Convert bytes to integers and display them.
            for (int i = 0; i < buffer.Length; i += BYTES_PER_INT) {
                int intValue = BitConverter.ToInt32(buffer, i);
                Console.Write(intValue + ", ");
            }
        }

        // Clear and size buffer to 4 bytes or remaining bytes if < 4.
        private static byte[] ResetBuffer(int fileLength, int totalBytesRead) {
            byte[] buffer = null;
            // Allocate space if data is left in file.
            if (totalBytesRead < fileLength) {
                if (totalBytesRead + BUFFER_SIZE <= fileLength)
                    buffer = new byte[BUFFER_SIZE];
                else
                    buffer = new byte[fileLength - totalBytesRead];
            }
            return buffer;
        }
    }
}
