/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace ConsoleApplication1 {
    class Program {
        private static void Main() {
            // Load the file.
            const string FILEPATH = "..\\..\\us_states.xml";
            XElement         root = XElement.Load(FILEPATH);
            
            // Query syntax.
            IEnumerable<XElement> querySyntax =
                from    e in root.Elements("state")
                orderby e.Element("capital").Element("city").Value descending
                where   e.Attribute("region").Value == "West"
               && (int) e.Element("population") > 1000000
                select  e;
            Display(querySyntax, "* Query Syntax *");
            
            // Method-based syntax.
            IEnumerable<XElement> methodBasedSyntax =
                root.Elements("state").OrderByDescending(
                           e => e.Element("capital").Element("city").Value)
                    .Where(e => e.Attribute("region").Value == "West"
                       && (int) e.Element("population") > 1000000);
            Display(methodBasedSyntax, "* Method Based Syntax *");
            Console.ReadLine();
        }

        private static void Display(IEnumerable<XElement> elements, string title) {
            Console.WriteLine(title);

            // Show the value of the 'name' element.
            foreach (XElement e in elements) {
                Console.Write(e.Element("capital").Element("city").Value);
                Console.Write(", " + e.Element("name").Value);
                Console.WriteLine(" (" + e.Element("name").Attribute("abbreviation")
                                                          .Value + ")");
            }
            Console.WriteLine();
        }
    }
}
