/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;

namespace ConsoleApplication1 {
    static class Program {
        public static void Main() {
            ShowMetadata(typeof (StartApp));
            ShowMetadata(typeof (EndApp));
            Console.ReadLine();
        }

        private static void ShowMetadata(Type objectType) {
            // Get all 'Help' attributes.
            var attributes = objectType.GetCustomAttributes(typeof (Help), true);

            // Iterate through 'Help attributes'.
            foreach (Attribute attribute in attributes) {
                // Cast each attribute as a 'Help' object.
                Help help = (Help) attribute;
                // Display detail and version of 'Help' attribute.
                if (null != help)
                    Console.WriteLine(help.Detail + " Version: "
                                      + help.Version);
            }
        }
    }

    // Define custom attribute.
    [AttributeUsage(AttributeTargets.Class)]
    public class Help : Attribute {
        public Help(string detail) {
            Detail  = detail;
            Version = "Undefined";
        }

        public string Detail  { get; protected set; }
        public string Version { get; set; }
    }

    // Class declarations with 'Help' attribute.
    [Help("Contains setup methods.")]
    public class StartApp {
    }

    [Help("Contains shutdown methods.", Version = "1.0")]
    public class EndApp {
    }
}
