/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;
using System.Reflection;

namespace ConsoleApplication1 {
    public class StartApp {
        // Fields, properties, constructors, and methods are all members.
        public int classLevelVariable = 5;    // Field
        public DateTime Started { get; set; } // Property

        public StartApp(DateTime started, int version) {
            // Constructor
            Started = started;
        }

        public void ShowStartTime() {
            // Method
            Console.WriteLine(Started);
        }
    }

    static class Program {
        public static void Main() {
            ShowMetadata(typeof (StartApp));
            Console.ReadLine();
        }

        private static void ShowMetadata(Type objectType) {
            FieldInfo[] fields = objectType.GetFields();            // Field
            Console.WriteLine("\n* Field data *");
            foreach (FieldInfo field in fields)
                Console.WriteLine(field.Name);

            PropertyInfo[] properties = objectType.GetProperties(); // Property
            Console.WriteLine("\n* Property data *");
            foreach (PropertyInfo property in properties)
                Console.WriteLine(property.Name);

            ConstructorInfo[] constructors = objectType.GetConstructors(); // Constructor
            Console.WriteLine("\n* Constructor data *");            
            foreach (ConstructorInfo constructor in constructors)
                Console.WriteLine(constructor.Name);

            MethodInfo[] methods = objectType.GetMethods();         // Method
            Console.WriteLine("\n* Method data *");
            foreach (MethodInfo method in methods)
                Console.WriteLine(method.Name);

            // Fields, properties, constructors, and methods are all members.
            MemberInfo[] members = objectType.GetMembers();
            Console.WriteLine("\n* Member data *");
            foreach (MemberInfo member in members)
                Console.WriteLine(member.Name);
        }
    }
}
