/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;
using System.Reflection;

namespace ConsoleApplication1 {
    public class BugFixInfo : Attribute {
        // Define custom attribute.
        public string ProgrammerName { get; set; }
        public int    BugID          { get; set; }

        public BugFixInfo(int bugID, string programmerName) {
            ProgrammerName = programmerName;
            BugID          = bugID;
        }
    }

    public class StartApp {
        [BugFixInfo(451, "Michael Abrash")] // Method.
        public StartApp(DateTime started) {
        }

        [BugFixInfo(488, "Grace Hopper")]   // Constructor.
        public void ShowStartTime() {
        }
    }

    internal static class Program {
        public static void Main() {
            // Extract Type for use with Reflection.
            Type    startApp = typeof (StartApp);
            string className = startApp.Name;

            // Obtain and iterate through all members.
            MemberInfo[] members = startApp.GetMembers();
            foreach (MemberInfo member in members) {
                const bool INHERITED = true;

                // Obtain and iterate through all 'BugFixInfo' attributes.
                var attributes = member.GetCustomAttributes(typeof (BugFixInfo), INHERITED);
                foreach (Attribute attribute in attributes) {
                    // Report on member and related bug fix data.
                    BugFixInfo bugfix = (BugFixInfo) attribute;
                    Console.WriteLine("\n" + className + ": "   + member.Name);
                    Console.WriteLine("Bug ID: "   + bugfix.BugID + " "
                                    + "Fixed by: " + bugfix.ProgrammerName);
                }
            }
            Console.ReadLine();
        }
    }
}
