/*********************************************************************
 * C#: A Beginner's Guide © 2015
 * McGraw-Hill Education
 * http://mhprofessional.com/computingdownload
 *********************************************************************/
using System;
using System.Reflection;
using System.Text.RegularExpressions;

namespace ConsoleApplication1 {
    internal static class Program {
        public static void Main() {
            ShowValidationStatus(new Person("John")); // Valid instance.
            ShowValidationStatus(new Person("7313")); // Invalid instance.
            Console.ReadLine();
        }

        private static void ShowValidationStatus(Person name) {
            bool valid = Validation.IsValid(name);
            Console.Write(name.First);
            if (valid)
                Console.WriteLine(" is valid.");
            else
                Console.WriteLine(" is not valid.");
        }
    }

    // Define custom attribute with optional AttributeUsage flags.
    [AttributeUsage(AttributeTargets.Property, // Target structure.
        AllowMultiple = false,                 // 1 instance allowed.
        Inherited     = true)]                 // Can't be inherited.
    public class MyRegex : Attribute {
        public string Pattern { get; private set; }

        public MyRegex(string pattern) {
            Pattern = pattern;
        }
    }

    public class Person {
        [MyRegex("^[a-zA-Z]+$")]               // Apply attribute to only permit
        public string First { get; set; }      // upper or lower case letters.

        public Person(string firstName) {
            First = firstName;
        }
    }

    public class Validation {
        private static bool CheckRegex(Object obj, PropertyInfo property) {
            // Select attributes & value where MyRegex attribute found.
            var attributes = property.GetCustomAttributes(typeof (MyRegex));
            
            // Iterate through 'MyRegex' attributes.
            foreach (Attribute attribute in attributes) {
                
                // Convert value with 'MyRegex' attribute to a string.
                var propertyValue = property.GetValue(obj, null);
                string personName = propertyValue.ToString();
                
                // Get pattern from MyRegex attribute.
                MyRegex MyRegex = (MyRegex) attribute;
                string pattern = MyRegex.Pattern;

                // Ensure property value conforms to the pattern.
                if (!Regex.IsMatch(personName, pattern))
                    return false;
            }
            return true;
        }

        public static bool IsValid(object obj) {
            var classType = obj.GetType();
            var properties = classType.GetProperties(); // Get properties.
            
            // Search properties and apply regex where 'MyRegex' applies.
            foreach (var property in properties) {
                if (!CheckRegex(obj, property))
                    return false;
            }
            return true;
        }
    }
}
